import warnings
import webbrowser
from wordcloud import WordCloud
import matplotlib.pyplot as plt
from gensim import corpora, models
import pyLDAvis.gensim_models
import pyLDAvis


def warn(*args, **kwargs):
    pass


warnings.warn = warn


class LDA:
    """
    performs Latent Dirichlet Allocation Topic Modelling
    """

    def __init__(self):
        pass

    @staticmethod
    def preperation(df_processed, no_below=None, no_above=None,
                    column="preprocessed"):
        """Preprocessing for LDA

        Args:
            df_processed (pd.DataFrame): DataFrame that stores tokenized text in
                col ["tokens"]
            no_below (int): if word appears in less than
                no_below documents they are not considered. Defaults to None.
            no_above (float): if e.g. 0.9, no words are taken into
                account that appear more often than 90%. Defaults to None.

        Returns:
            dictionary: [returns the dictionary used for LDA]
            bow_corpus: [Corpus used for LDA]
        """

        dictionary = corpora.Dictionary(df_processed[column])
        if no_below is None:
            no_below = 0
        if no_above is None:
            no_above = 1
        bow_corpus = [dictionary.doc2bow(doc) for doc in df_processed[column]]
        return dictionary, bow_corpus

    @staticmethod
    def model(
        corpus,
        num_topics,
        id2word,
        random_state=101,
        passes=20,
        chunksize=500,
        eta="auto",
        eval_every=None,
        multi=True,
        alpha="asymmetric",
    ):
        """LDA model

        Args:
            corpus (iterable of list of (int, float), optional): Stream of
                document vectors or sparse matrix of shape
            num_topics (int): pre-defined number of topics
            id2word ({dict of (int, str):
                gensim.corpora.dictionary.Dictionary}) –Mapping from word IDs to
                words. It is used to determine the vocabulary size, as well as
                for debugging and topic printing.
            random_state (int): for recreating exact identical output. Defaults
                to 101.
            passes (int): Number of passes through the corpus during training.
                Defaults to 20.
            chunksize (int, optional): chunksize in lda passes. Defaults to 500.
            eta (str, optional): [description]. Defaults to "auto".
            eval_every ([type], optional): Hyperparameter in LDA used to
                initiliaze the Dirichlet distribution. Defaults to None.
            multi (bool, optional): If true, the in gensim incorporated
                multicore variant is used. Defaults to True.
            alpha (str, optional): OTher Dirichlet Prior. Defaults to
                "asymmetric".

        Returns:
            lda_model: returns lda_model output
        """

        if multi is True:
            lda_model = models.LdaMulticore(
                random_state=random_state,
                alpha=alpha,
                corpus=corpus,
                num_topics=num_topics,
                id2word=id2word,
                passes=passes,
                chunksize=chunksize,
                eta=eta,
                eval_every=eval_every,
            )

        else:
            lda_model = models.LdaModel(
                random_state=random_state,
                alpha=alpha,
                corpus=corpus,
                num_topics=num_topics,
                id2word=id2word,
                passes=passes,
                chunksize=chunksize,
                eta=eta,
                eval_every=eval_every,
            )

        return lda_model

    @staticmethod
    def visualize_topics(
        lda_model,
        bow_corpus,
        dictionary,
        save_name="audolab_model.png",
        type="clouds",
        figsize=(50, 30),
        facecolor="k",
        width=2000,
        height=1000,
        background_color="white",
        topic=0,
        words=100,
        save=False,
        n_clouds=1,
    ):
        """Visualizes the topic models output in wordclouds or pyldavis

        Args:
            lda_model (gensim_models.ldamodel.LdaModel): the created LDA model
            bow_corpus (gensim.corpora.dictionary.Dictionary): Bag of words
                corpus of used documents
            dictionary (gensim.corpora.dictionary.Dictionary): Dictionary of
                all words
            save_name (str, optional): name under which the plots should be
                save. Defaults to "audolab_model.png".
            type (str, optional): type of visualisation- either "clouds" or
                "pyldavis". Defaults to "clouds".
            figsize (tuple, optional): Size of wordclouds. Defaults to (50, 30).
            facecolor (str, optional): Colour of wordcloud Defaults to "k".
            width (int, optional): width of plots. Defaults to 2000.
            height (int, optional): height of plots. Defaults to 1000.
            background_color (str, optional): Background colour of wordcloud.
                Defaults to "white".
            topic (int, optional): IF only one wordcloud is plotted, index of
                topic that is plotted. Defaults to 0.
            words (int, optional): Number of words per cloud. Defaults to 100.
            save (bool, optional): whether the plots should be saved or not.
                Defaults to False.
            n_clouds (int, optional): Number of word clouds that are plotted.
                Defaults to 1.

        Raises:
            ValueError: If save_name is not a string: no "Please specify a
                string as the name under which the plots should be saved"
        """

        if type == "pyldavis":
            # siehe: https://pyldavis.readthedocs.io/en/latest/modules/API.html
            # doc_lengths = list(
            #   preprocessed_column_target.apply(lambda x: len(x))
            #   )
            visualization = pyLDAvis.gensim_models.prepare(
                lda_model,
                bow_corpus,
                dictionary,
                sort_topics=False
            )
            pyLDAvis.save_html(
                visualization,
                'your_latest_lda_visualization.html')
            webbrowser.open('your_latest_lda_visualization.html')

        if type == "clouds" and n_clouds <= 1:
            plt.figure(figsize=figsize, facecolor=facecolor)
            plt.imshow(
                WordCloud(
                    width=width,
                    height=height,
                    background_color=background_color
                ).fit_words(dict(lda_model.show_topic(topic, words)))
            )
            plt.axis("off")
            plt.title("Lda Model " + "topic #" + str(topic))
            plt.tight_layout(pad=0)

            plt.show()

        def _wordclouds(topic):
            wordcloud = WordCloud(
                width=width, height=height, background_color=background_color
            ).fit_words(dict(lda_model.show_topic(topic, words)))
            return wordcloud

        if type == "clouds" and n_clouds >= 1:
            fig = plt.figure()
            if n_clouds >= 10:
                for i in range(n_clouds):
                    ax = fig.add_subplot(
                        int(n_clouds / 3), int(n_clouds / 3), i + 1)
                    wordcloud = _wordclouds(i)
                    ax.imshow(wordcloud)
                    ax.axis("off")
            else:
                for i in range(n_clouds):
                    ax = fig.add_subplot(2, int(n_clouds / 2), i + 1)
                    wordcloud = _wordclouds(i)
                    ax.imshow(wordcloud)
                    ax.axis("off")

            plt.show()

        if save:
            if not isinstance(save_name, str):
                raise ValueError(
                    "Please specify a string as the name under which the plots should be saved"
                )

            plt.savefig(save_name)



import re
from bs4 import BeautifulSoup
import numpy as np
import pandas as pd
from requests import get as requests_get
from json import loads as json_loads
import asyncio
from pyppeteer import launch
from tqdm import tqdm


class AbstractScraper:
    """
    Abstractscraper - scrapes through IEEE Xplore and, depending on the given
    search query, scrapes the abstracts of scientific papers
    """

    def __init__(self):
        self.first = "https://ieeexplore.ieee.org/search/searchresult.jsp?action=search&newsearch=true&matchBoolean=true&"
        self.second = "&highlight=true&returnFacets=ALL&returnType=SEARCH&matchPubs=true&rowsPerPage=100&pageNumber=1"

    async def _open(
        self, url=None, keywords=None, operator="OR", in_data="author", pages=2
    ):
        """
        defines the users search query

        :param url: when the user specifies an own search query on IEEEXplore
        :type url: string
        :param keywords: keywords that are searched for
        :type keywords: list
        :param operator: "and" / "or" operator between keywords
        :type operator: string
        :param pages: number of pages that is iterated over
        :type pages: int
        :param in_data: "author" or "all_meta" whether to search in author
                         keywords or all metadata
        :type in_data: string
        """

        await self._open_pypeteer(url, keywords, operator, in_data, pages)

    async def _open_pypeteer(
        self, url=None, keywords=None, operator="OR", in_data="author", pages=2
    ):
        """[summary]

        Args:
            url (string, optional): if given than the search defined in this url
                is scraped. Defaults to None.
            keywords (list, optional): f no url is given but a keyword list, a
                search after these keywords is performed and the results are
                scraped. Defaults to None.
            operator (str, optional): Operator between the keywords. If "AND"
                the search results must include all specified keywords. Defaults
                to "OR".
            in_data (str, optional): "author" or "all_meta" -> whether the
                keywords are only searched in the author keywords or in all
                metadata. Defaults to "author".
            pages (int, optional): number of pages that are iterated over.
                Defaults to 2.
        """

        self._create_url(url, keywords, operator, in_data)
        self.html_page = []
        print("The algorithm is iterating through", pages, "pages")
        browser = await launch()
        page = await browser.newPage()
        await page.goto(self.url, {"waitUntil": "networkidle0"})

        self.linkstorage = []

        elements = await page.querySelectorAll("a")
        for element in elements:
            self.linkstorage.append(
                await page.evaluate("(element) => element.href", element)
            )
        for i in range(2, pages + 1):
            self._create_new_url(i)
            await page.goto(self.url, {"waitUntil": "networkidle0"})
            elements = await page.querySelectorAll("a")
            for element in elements:
                self.linkstorage.append(
                    await page.evaluate("(element) => element.href", element)
                )
        await browser.close()

    def _add_page_number(self, page, position):
        """[summary]

        Args:
            page ([type]): [description]
            position ([type]): [description]
        """
        self.url = (
            self.url[: position + len("pageNumber=")]
            + str(page)
            + self.url[position + len("pageNumber=") + len(str(page - 1)):]
        )

    def _add_str_plus_page_number(self, page):
        """[summary]

        Args:
            page ([type]): [description]
        """
        if self.url[-1] == "?":
            self.url = self.url + "pageNumber=" + str(page)
        else:
            self.url = self.url + "?pageNumber" + str(page)

    def _create_new_url(self, page):
        """[summary]

        Args:
            page ([type]): [description]
        """

        if "pageNumber=" in self.url:
            self._add_page_number(page, self.url.find("pageNumber="))
        else:
            self._add_str_plus_page_number(page)

    def _create_url(self, url, keywords, operator, in_data):
        """[summary]

        Args:
            url ([type]): [description]
            keywords ([type]): [description]
            operator ([type]): [description]
            in_data ([type]): [description]
        """

        operator = ")%20" + operator.upper() + "%20("
        if in_data == "all_meta":
            in_data = "%22All%20Metadata%22:"
        elif in_data == "author":
            in_data = "%22Author%20Keywords%22:"
        if url:
            self.url = url

        else:
            if len(keywords) > 3:
                print("You can only specify 3 Keywords/phrases")
            elif len(keywords) == 1:
                self.middle = "queryText=" + in_data + keywords[0]
            elif len(keywords) == 2:
                self.middle = (
                    "queryText=(("
                    + in_data
                    + keywords[0]
                    + operator
                    + in_data
                    + keywords[1]
                    + "))"
                )
            else:
                self.middle = (
                    "queryText=(("
                    + in_data
                    + keywords[0]
                    + operator
                    + in_data
                    + keywords[1]
                    + operator
                    + in_data
                    + keywords[2]
                    + "))"
                )
            self.url = self.first + self.middle + self.second

    def _find_links(self):
        """goes through every paper on every page and collects all links to the
        subpages of the papers"""

        self.data = []
        document = "/document/"
        citation = "/citations"
        for link in self.linkstorage:
            if (document in link) & (citation not in link):
                self.data.append(link)
        # remove unnecessary results of the href search
        self.data = np.array(self.data)
        # remove duplicates that are in there due to multiple occurrence in the
        self.data = np.unique(self.data)
        print("Total number of abstracts that will be scraped:", len(self.data))

    def _find_abstracts(
        self,
        features=[
            "abstract",
            "title",
            "citationCount",
            "doi",
            "totalDownloads",
            "keywords",
            "publicationYear",
        ],
    ):
        """Opens all links for the webpages for each paper and scrapes the
        paper's abstract

        Args:
            features (list, optional): specify which information you want to
                have scraped. Defaults to ["abstract", "title",
                "citationCountPaper", "doi", "totalDownloads"].
        """

        # initialize emtpy dict to create attributes and set dict keys as
        # features input and values as empty lists
        att_creator = {}
        for i in range(len(features)):
            att_creator[features[i]] = []

        # cretae the object attributes based on the input in list (features)
        for i, j in att_creator.items():
            setattr(self, i, j)

        # loop through number of  "link"
        for i in tqdm(range(len(self.data))):
            # only "try" because sometimes the javascript is corrupted
            # get the html data of the webpage with metadata such as abstracts
            # titles etc.
            try:
                data = json_loads(
                    re.search(
                        r"\.metadata=(.*?);",
                        requests_get(self.data[i]).text,
                    ).group(1)
                )
            except BaseException:
                pass

            # loop through list of desired features and extract information
            for i in features:
                # if the data is not available, e.g. the paper has no
                # citations, use None
                try:
                    temp = data[i]
                except BaseException:
                    temp = None

                # for views the data is differently extracted
                if i == "totalDownloads":
                    try:
                        temp = data["metrics"][i]
                    except BaseException:
                        temp = None
                if i == "keywords":
                    try:
                        temp = data[i][0]["kwd"]
                    except BaseException:
                        temp = None
                # append to obj attribute
                getattr(self, i).append(temp)

    async def get(
        self, url=None, keywords=None, operator="OR", pages=2, in_data="author"
    ):
        """simply runs the self._open function that open ieeeXplore and seraches
            for paper urls

        Args:
            url (str, optional): when the user specifies an own search query on
                IEEEXplore. Defaults to None.
            keywords (iist, optional): keywords that are searched for. Defaults
                to None.
            operator (str, optional): [description]. Defaults to "OR".
            pages (int, optional): Number of pages the algorithm iterates over.
                Defaults to 2.
            in_data (str, optional): "author" or "all_meta" whether to search in
                author keywords or all metadata. Defaults to "author".
        """
        await self._open(
            url=url,
            keywords=keywords,
            operator=operator,
            pages=pages,
            in_data=in_data
        )

    async def get_abstracts(
        self,
        url=None,
        keywords=None,
        operator="OR",
        pages=2,
        in_data="author",
        features=[
            "abstract",
            "title",
            "citationCount",
            "doi",
            "totalDownloads",
            "keywords",
            "publicationYear",
        ],
    ):
        """Function to scrape abstracts of scientific papers from the givin url.

        We used https://ieeexplore.ieee.org/search/advanced to generate a
        list like https://ieeexplore.ieee.org/search/searchresult.jsp?action=se
        arch&newsearch=true&matchBoolean=true&queryText=(%22Author%20Keywords%22
        :cotton)&highlight=true&returnFacets=ALL&returnType=SEARCH&matchPubs=Tru
        e&rowsPerPage=100&pageNumber=1
        with the search results.
        The abstracts of the papers listet on that list of search results will
        be stored in a .txt file with the givin file name.

        Args:
            url (str, optional): when the user specifies an own search query on
                IEEEXplore. Defaults to None.
            keywords (iist, optional): keywords that are searched for. Defaults
                to None.
            operator (str, optional): [description]. Defaults to "OR".
            pages (int, optional): Number of pages the algorithm iterates over.
                Defaults to 2.
            in_data (str, optional): "author" or "all_meta" whether to search in
                author keywords or all metadata. Defaults to "author".
            features (list, optional): which features should be scraped.
                Defaults to ["abstract", "title", "citationCount", "doi",
                "totalDownloads", "keywords", "publicationYear"].

        Returns:
            pd.DataFrame: DataFrame containing the paper abstracts and the
                selected features
        """

        await self._open(
            url=url,
            keywords=keywords,
            operator=operator,
            pages=pages,
            in_data=in_data
        )
        self._find_links()
        self._find_abstracts(features=features)

        data = pd.DataFrame()
        for i in features:
            data[i] = getattr(self, i)

        data = data.drop_duplicates(subset=["abstract", "title"])
        return data


if __name__ == "__main__":

    async def main():
        AS = AbstractScraper()
        data = await AS.get_abstracts(
            url="https://ieeexplore.ieee.org/search/searchresult.jsp?highlight=true&returnType=SEARCH&matchPubs=true&rowsPerPage=100&sortType=paper-citations&returnFacets=ALL&pageNumber=1",
            pages=1,
        )

        return data

    data = asyncio.get_event_loop().run_until_complete(main())

    data.to_csv("cit_patents.csv")



import warnings
with warnings.catch_warnings():
    warnings.filterwarnings("ignore", category=DeprecationWarning)
    import re
    import nltk
    from nltk.tokenize import RegexpTokenizer
    from nltk.corpus import stopwords
    from gensim.models import Phrases
    from tqdm import tqdm


# Text preparation


class Preprocessor:
    def __init__(self):
        4 + 5

    def _text_prepare(self, text, language="english", stop_words=True):
        """text preparation function for text preprocessing

        Args:
            text (helper variable): None
            language (str, optional): Sets language of stopwords to be removed.
                Defaults to "english".
            stop_words (bool, optional): If true, the stopwords are removed, if
                not, stopwords are left as they are. Defaults to True.

        Returns:
            [type]: [description]
        """

        REPLACE_BY_SPACE_RE = re.compile(r"[/(){}\[\]\|@,;]")
        BAD_SYMBOLS_RE = re.compile("[^0-9a-z #+_]")
        NUMBERS = re.compile(r"\d+")

        text = text.lower()
        text = REPLACE_BY_SPACE_RE.sub(
            "", text
        )  # replace REPLACE_BY_SPACE_RE symbols by space in text

        text = BAD_SYMBOLS_RE.sub("", text)
        text = NUMBERS.sub("", text)
        # delete symbols which are in BAD_SYMBOLS_RE from text
        words = text.split()

        if stop_words:
            i = 0
            STOPWORDS = set(stopwords.words(language))
            while i < len(words):
                if words[i] in STOPWORDS:
                    words.pop(i)
                else:
                    i += 1
        text = " ".join(map(str, words))  # delete stopwords from text

        return text

    def _lemmatize_text(self, text):
        """helper function that lemmatizes already tokenized text"""

        lemmatizer = nltk.stem.WordNetLemmatizer()
        return " ".join([lemmatizer.lemmatize(w, "v") for w in text])

    def _preprocessing(self, df, column):
        """Preprocessing function that calls the helper functions

        :param df: DataFrame that has the text data stored
        :type df: pd.DataFrame
        :param column: column name where raw text is stored
        :type column: str

        :return: DataFrame with preprocessed text
        :rtype:  DataFrame
        """

        df_temp = df.copy(deep=True)
        df_temp[column] = df_temp[column].astype(str)

        df_temp.loc[:, column] = [self._text_prepare(
            x) for x in df_temp[column].values]
        df_temp[column] = [
            item for item in df_temp[column] if not item.isdigit()]

        tokenizer = RegexpTokenizer(r"\w+")

        df_temp["tokens"] = df_temp[column].apply(tokenizer.tokenize)
        df_temp["lemma"] = df_temp["tokens"].apply(self._lemmatize_text)
        df_temp["tokens"] = df_temp["lemma"].apply(tokenizer.tokenize)

        return df_temp, df_temp["tokens"]

        # apply prepro func

    def basic_preprocessing(self, df, column, ngram_type=2):
        """The data will be lemmatized, tokenized and the stopwords will be
        deleted.

        Args:
            df (pd.DataFrame): Dataframe where the documents to be preprocessed
                are stored.
            column (str):  Column name of the column where docs are stored.
            ngram_type (int, optional): Number of ngrams used. Defaults to 2.

        Returns:
            pd.DataFrame: DataFrame where the original docus and the
                preprocessed documents are stored.
        """
        df, df_txt = self._preprocessing(df=df, column=column)
        df_txt = df_txt.reset_index()
        df_txt = df_txt.drop("index", axis=1)
        df_txt = df_txt["tokens"]

        df = df.reset_index()
        df = df.drop("index", axis=1)

        if ngram_type == 2:
            bigram = Phrases(df_txt, min_count=10)

            for idx in tqdm(range(len(df_txt))):

                for token in bigram[df_txt[idx]]:
                    if "_" in token:
                        df_txt[idx].append(token)

        if ngram_type == 3:
            bigram = Phrases(df_txt, min_count=10)
            trigram = Phrases(bigram[df_txt])

            for idx in tqdm(range(len(df_txt))):
                for token in bigram[df_txt[idx]]:
                    if "_" in token:
                        df_txt[idx].append(token)
                for token in trigram[df_txt[idx]]:
                    if "_" in token:
                        df_txt[idx].append(token)

        if ngram_type == 4:
            bigram = Phrases(df_txt, min_count=10)
            trigram = Phrases(bigram[df_txt])
            fourgram = Phrases(trigram[df_txt])

            for idx in tqdm(range(len(df_txt))):
                for token in bigram[df_txt[idx]]:
                    if "_" in token:
                        df_txt[idx].append(token)
                for token in trigram[df_txt[idx]]:
                    if "_" in token:
                        df_txt[idx].append(token)
                for token in fourgram[df_txt[idx]]:
                    if "_" in token:
                        df_txt[idx].append(token)

        if ngram_type >= 4:
            print("please specify a ngram_type <= 4")

        df["preprocessed"] = df_txt
        df["lemma"] = [" ".join(map(str, j)) for j in df["preprocessed"]]

        df = df.drop(["tokens"], axis=1)

        return df


if __name__ == "__main__":
    import pandas as pd

    data = pd.read_csv("mtsamples.csv")
    prepro = Preprocessor()
    test = prepro.basic_preprocessing(
        df=data, column="transcription", ngram_type=3)



import warnings
import asyncio
import sys
from AuDoLab.subclasses import abstractscraper_pubmed
from AuDoLab.subclasses import abstractscraper_arxiv
from AuDoLab.subclasses import tf_idf
from AuDoLab.subclasses import preprocessing
from AuDoLab.subclasses import one_class_svm
from AuDoLab.subclasses import lda
from AuDoLab.subclasses import abstractscraper
import pandas as pd


def warn(*args, **kwargs):
    pass


warnings.warn = warn


class AuDoLab:

    def __init__(self):
        self.loop = asyncio.get_event_loop()
        if self._is_notebook():
            # prevent runtime error with asyncio in ipynb:
            # https://medium.com/@vyshali.enukonda/how-to-get-around-runtimeerro
            # r-this-event-loop-is-already-running-3f26f67e762e
            import nest_asyncio
            nest_asyncio.apply()

    def get_ieee(
        self,
        url=None,
        keywords=None,
        operator="OR",
        pages=2,
        in_data="author",
        prepro=False,
        ngram_type=2,
    ):
        """Function to scrape abstracts of scientific papers from the givin url.

        We used https://ieeexplore.ieee.org/search/advanced to generate a
        list like https://ieeexplore.ieee.org/search/searchresult.jsp?action=se
        arch&newsearch=true&matchBoolean=true&queryText=(%22Author%20Keywords%22
        :cotton)&highlight=true&returnFacets=ALL&returnType=SEARCH&matchPubs=Tru
        e&rowsPerPage=100&pageNumber=1
        with the search results.
        The abstracts of the papers listet on that list of search results will
        be stored in a .txt file with the givin file name.

        Args:
            url (str, optional): The url of the website, whos presented paper
                abstracs will be scraped. Defaults to None.
                keywords (list, optional): List of keywords that are searched
                for. Defaults to None.
            keywords (iist, optional): keywords that are searched for.
                Defaults to None.
            operator (str, optional): Operator between the keywords.
                "AND" or "OR". If "AND" the search results must include all
                keywords. Defaults to "OR".
            pages (int, optional): Number of pages that are iterated over.
                Translates directly to number of abstracts that are scraped.
                Roughly there are 100 abstracts scraped per page. Defaults to 2.
            in_data (str, optional): If the keywords are searched for in the
                author keywords or in all metadata. Defaults to "author".
            prepro (bool, optional): if True, the scraped data will directly be
                preprocessed for later use. Defaults to False.
            ngram_type (int, optional): number of ngrams in preprocessing.
                Defaults to 2.

        Returns:
            pd.DataFrame: DataFrame with the stored abstracts and metadata
        """
        return self.loop.run_until_complete(self.__async__get_ieee(
            "https://ieeexplore.ieee.org/search/searchresult.jsp?newsearch=true&queryText=cotton&highlight=true&returnFacets=ALL&returnType=SEARCH&matchPubs=true&rowsPerPage=100&pageNumber=1",
            prepro=True,
            pages=pages,
        ))

    async def __async__get_ieee(
        self,
        url=None,
        keywords=None,
        operator="OR",
        pages=2,
        in_data="author",
        prepro=False,
        ngram_type=2,
    ):

        number = pages

        ks = abstractscraper.AbstractScraper()
        self.abstracts = await ks.get_abstracts(
            url=url,
            keywords=keywords,
            operator=operator,
            pages=number,
            in_data=in_data
        )

        if prepro is True:
            self.abstracts = self.abstracts.reset_index(drop=True)
            self.abstracts = self.text_cleaning(
                self.abstracts, "abstract", ngram_type=ngram_type
            )

        if not isinstance(self.abstracts, pd.DataFrame):
            print(
                "if using the ieee abstractscraper, please use the following code: \n \n"
                + "async def scrape():"
                + "\n     return await audo.ieee_scraper(keywords=[keywords], prepro=False, pages=1, ngram_type=2)"
                + "\n\nscraped_documents = asyncio.get_event_loop().run_until_complete(scrape())"
            )

            sys.exit(
                "please specify the code as indicated above, or use the function abstract_scraper to scrape from different websites"
            )

        return self.abstracts

    def abstract_scraper(
        self, type="arxiv", url=None, pages=2, prepro=False, ngram_type=2
    ):
        """Scrapes the pages arxiv.org, pubmed.gov for paper abstracts

        Args:
            type (str, optional): "arxiv" or "pubmed". Defines for which page
                the scraping is done. Defaults to "arxiv".
            url (str, optional): The given url after which the papers are
                scraped. Must be in line with type. Defaults to None.
            pages (int, optional): Number of pages that are iterated over.
                Defaults to 2.
            prepro (bool, optional): If True, the scraped documents are
            preprocessed directly. Defaults to False.
            ngram_type (int, optional): Number of ngrams in preprocessing.
                Defaults to 2.

        Returns:
            pd.DataFrame: DataFrame with the stored abstracts
        """

        if type == "arxiv":
            ks = abstractscraper_arxiv.AbstractScraper_Arxiv()
            self.abstracts = ks.scrape_arxiv(url, pages)

        elif type == "pubmed":
            ks = abstractscraper_pubmed.AbstractScraper_Pubmed()
            self.abstracts = ks.scrape_pubmed(url, pages)

        self.abstracts = self.abstracts.reset_index(drop=True)

        if prepro is True:
            self.abstracts = self.text_cleaning(
                self.abstracts, "abstract", ngram_type=ngram_type
            )

        return self.abstracts

    def text_cleaning(self, data, column, ngram_type=2):
        """The data will be lemmatized, tokenized and the stopwords will be
        deleted.

        Args:
            data (pd.DataFrame): Dataframe where the documents to be
                preprocessed are stored
            column (str): Column name of the column where docs are stored
            ngram_type (int, optional): Number of ngrams used. Defaults to 2.

        Returns:
            pd.DataFrame: DataFrame where the original docus and the
                preprocessed documents are stored
        """

        prepro = preprocessing.Preprocessor()
        self.data_processed = prepro.basic_preprocessing(
            data, column, ngram_type=ngram_type
        )
        return self.data_processed

    def tf_idf(
        self,
        data,
        papers,
        data_column,
        papers_column,
        features=None,
        ngrams=2
    ):
        """Creates tf-idf objects for one-class SVM classification.

        The tf-idf scores are calculated over a joint corpus, however the target
        data and the out-of-domain training data are stored in seperate, as the
        one-class SVM is only trained on the tf-idf scores of the out-of-domain
        training data.

        Args:
            data (DataFrame): preprocessed target documents
            papers (DataFrame): preprocessed out-of-domain training data
            data_colum (String): name of columnin target dataframe where
                lemmatized documents are stored. Defaults to 'lemma'
            papers_colum (String): name of column in out-of-domain training
                dataframe where lemmatized documents are stored. Defaults to
            'lemma' ngrams (int, optional): whether ngram are formed.
                Defaults to 2.
            features (int, optional): number of max features.
                Defaults to 8000.

        Returns:
            data and papers: tfidf object data for target data and
                out-of-domain training data
        """

        tfidf = tf_idf.Tf_idf()
        self.data_tfidf, self.papers_tfidf = tfidf.tfidf(
            data, papers, data_column, papers_column, features, ngrams
        )
        return self.data_tfidf, self.papers_tfidf

    def one_class_svm(
        self,
        training,
        predicting,
        nus,
        quality_train=0.85,
        min_pred=0.05,
        max_pred=0.2,
        gamma="auto",
        kernel="rbf",
    ):
        """Returns the classifiers that fullfill the required conditions.

        Args:
            training (DataFrame): training dataset of preprocessed documents
            predicting (DataFrame): target dataset of preprccessed documents
            nus (list of floats): hyperparameters over which are looped. For
                each nu the classifier is trained
            quality_train (float, optional): percentage of training data that
                seems to belong to target class. Default: 0.85. Defaults to
                0.85.
            min_pred (float, optional): percentage of target data that has to be
                at least classified as belonging to target class for classifier
                to be considered. Default: 0.0. Defaults to 0.05.
            max_pred (float, optional): percentage of target class that is
                maximally allowed to be classified as belonging to
            target class for classifier to be considered.. Defaults to 0.2.
            gamma (str, optional): Hyperparamter of O-SVM. Defaults to "auto".
            kernel (str, optional): Kernel function used in O_SVM. Defaults to
                "rbf".

        Returns:
            pd.DataFrame: DataFrame with stored classifiers that fulfill
                conditions
        """

        one_Class_SVM = one_class_svm.One_Class_SVM()
        self.df = one_Class_SVM.classification(
            training=training,
            predicting=predicting,
            nus=nus,
            quality_train=quality_train,
            min_pred=min_pred,
            max_pred=max_pred,
            gamma=gamma,
            kernel=kernel,
        )

        return self.df

    def choose_classifier(self, df, classifier, i):
        """Returns dataframe where documents that are classified to target class
        have 1, otherwise, 0

        Args:
            df (pd.Dataframe): dataframe of target documents
            classifier (list): list of all possible o-svm classifiers
            i (int): index of which classifier is chosen/preferred

        Returns:
            pd.dataframe: documents that are classified as belonging to target
        """
        return one_class_svm.One_Class_SVM.choose_classifier(df, classifier, i)

    def lda_modeling(
        self,
        data,
        num_topics,
        corpus=None,
        dict=None,
        no_below=None,
        no_above=None,
        random_state=101,
        passes=20,
        chunksize=500,
        eta="auto",
        eval_every=None,
        multi=True,
        alpha="asymmetric",
        column="preprocessed",
    ):
        """The function performs lda modelling as described in this
        https://www.jmlr.org/papers/volume3/blei03a/blei03a.pdf paper.

        Args:
            corpus (iterable of list of (int, float), optional): Stream of
                document vectors or sparse matrix of shape
            num_topics (int): pre-defined number of topics
            id2word ({dict of (int, str): gensim.corpora.dictionary.Dictionary})
                –Mapping from word IDs to words. It is used to determine the
                vocabulary size, as well as for debugging and topic printing.
            random_state (int): for recreating exact identical output. Defaults
                to 101.
            passes (int): Number of passes through the corpus during training.
                Defaults to 20.
            chunksize (int, optional): chunksize in lda passes. Defaults to 500.
            eta (str, optional): [description]. Defaults to "auto".
            eval_every ([type], optional): Hyperparameter in LDA used to
                initiliaze the Dirichlet distribution. Defaults to None.
            multi (bool, optional): If true, the in gensim incorporated
                multicore variant is used. Defaults to True.
            alpha (str, optional): OTher Dirichlet Prior. Defaults to
                "asymmetric".

        Returns:
            lda_model: returns lda_model output
        """

        if corpus is None:
            self.dictionary, self.bow_corpus = lda.LDA.preperation(
                data, no_below, no_above, column=column
            )

        else:
            self.corpus = (corpus,)
            self.dictionary = dict

        self.lda = lda.LDA()
        self.lda_model = self.lda.model(
            corpus=self.bow_corpus,
            num_topics=num_topics,
            id2word=self.dictionary,
            random_state=random_state,
            passes=passes,
            chunksize=chunksize,
            eta=eta,
            eval_every=eval_every,
            multi=multi,
            alpha=alpha,
        )
        return self.lda_model

    def _is_notebook(self):
        try:
            shell = get_ipython().__class__.__name__
            if shell == 'ZMQInteractiveShell':
                return True   # Jupyter notebook or qtconsole
            elif shell == 'TerminalInteractiveShell':
                return False  # Terminal running IPython
            else:
                return False  # Other type (?)
        except NameError:
            return False      # Probably standard Python interpreter

    def lda_visualize_topics(
        self,
        save_name="audolab_model.png",
        lda_model=None,
        bow_corpus=None,
        dictionary=None,
        type="pyldavis",
        figsize=(20, 10),
        facecolor="k",
        width=1600,
        height=800,
        background_color="white",
        topic=0,
        words=100,
        save=False,
        n_clouds=1,
    ):
        """Visualizes the topic models output in wordclouds or pyldavis

        Args:
            lda_model (gensim.models.ldamodel.LdaModel): the created LDA model
            bow_corpus (gensim.corpora.dictionary.Dictionary): Bag of words corpus of used documents
            dictionary (gensim.corpora.dictionary.Dictionary): Dictionary of all words
            save_name (str, optional): name under which the plots should be save. Defaults to "audolab_model.png".
            type (str, optional): type of visualisation- either "clouds" or "pyldavis". Defaults to "clouds".
            figsize (tuple, optional): Size of wordclouds. Defaults to (50, 30).
            facecolor (str, optional): Colour of wordcloud Defaults to "k".
            width (int, optional): width of plots. Defaults to 2000.
            height (int, optional): height of plots. Defaults to 1000.
            background_color (str, optional): Background colour of wordcloud. Defaults to "white".
            topic (int, optional): IF only one wordcloud is plotted, index of topic that is plotted. Defaults to 0.
            words (int, optional): Number of words per cloud. Defaults to 100.
            save (bool, optional): whether the plots should be saved or not. Defaults to False.
            n_clouds (int, optional): Number of word clouds that are plotted. Defaults to 1.

        Raises:
            ValueError: If save_name is not a string: no "Please specify a string as the name under which the plots should be saved"
        """

        if bow_corpus is None:
            bow_corpus = self.bow_corpus

        if dictionary is None:
            dictionary = self.dictionary

        if lda_model is None:
            lda_model = self.lda_model

        lda.LDA.visualize_topics(
            lda_model,
            bow_corpus=bow_corpus,
            dictionary=dictionary,
            save_name=save_name,
            type=type,
            figsize=figsize,
            facecolor=facecolor,
            width=width,
            height=height,
            background_color=background_color,
            topic=topic,
            words=words,
            save=save,
            n_clouds=n_clouds,
        )



from sklearn.svm import OneClassSVM
from pandas import DataFrame


class One_Class_SVM:
    def __init__(self):
        3 + 4

    @staticmethod
    def choose_classifier(df, classifier, i):
        """Returns dataframe where documents that are classified to target class
        have 1, otherwise, 0

        Args:
            df (pd.Dataframe): dataframe of target documents
            classifier (list): list of all possible o-svm classifiers
            i (int): index of which classifier is chosen/preferred

        Returns:
            pd.dataframe: documents that are classified as belonging to target
        """
        return df.iloc[classifier.index[classifier.iloc[:, i] == 1].tolist()]

    @staticmethod
    def classification(
        training,
        predicting,
        nus,
        quality_train=0.85,
        min_pred=0.05,
        max_pred=0.2,
        gamma="auto",
        kernel="rbf",
    ):
        """Returns the classifiers that fullfill the required conditions.

        Args:
            training (DataFrame): training dataset of preprocessed documents
            predicting (DataFrame): target dataset of preprccessed documents
            nus (list of floats): hyperparameters over which are looped. For
                each nu the classifier is trained
            quality_train (float, optional): percentage of training data that
                seems to belong to target class. Default: 0.85. Defaults to
                0.85.
            min_pred (float, optional): percentage of target data that has to be
                at least classified as belonging to target class for classifier
                to be considered. Default: 0.0. Defaults to 0.05.
            max_pred (float, optional): percentage of target class that is
                maximally allowed to be classified as belonging to
            target class for classifier to be considered.. Defaults to 0.2.
            gamma (str, optional): Hyperparamter of O-SVM. Defaults to "auto".
            kernel (str, optional): Kernel function used in O_SVM. Defaults to
                "rbf".

        Returns:
            pd.DataFrame: DataFrame with stored classifiers that fulfill
                conditions
        """

        df = DataFrame()
        for i in nus:

            svm = OneClassSVM(nu=i, gamma=gamma, kernel=kernel)
            # fit the model for each kernel
            clf = svm.fit(training)

            train = clf.predict(training)

            if sum(train[train == 1]) >= round(quality_train * len(train)):

                prediction = clf.predict(predicting)

                if (
                    sum(prediction[prediction == 1])
                    >= round(min_pred * (predicting.shape[0]))
                ) and (
                    sum(prediction[prediction == 1])
                    <= round(max_pred * (predicting.shape[0]))
                ):
                    print(
                        "nu:",
                        str(i),
                        "data predicted:",
                        sum(prediction[prediction == 1]),
                        "training_data predicted:",
                        sum(train[train == 1]),
                    )

                    df["prediction: nu: ", str(i)] = prediction

        return df



# %load_ext nb_black
import pandas as pd


data = pd.read_csv(r"tests/mtsamples.csv")

data[data["medical_specialty"] == " Dentistry"]

data = data.sort_values("medical_specialty")


new_list = list(
    data[data["medical_specialty"] == "Dentistry"]["transcription"]
)


data["dentistry"] = data["transcription"].map(
    lambda x: 1 if x in new_list else -1
)
data = data.drop_duplicates(
    subset="transcription"
)  # , 'medical_specialty'], keep="first")



data = data.drop(data[data["transcription"].isna()].index)

data = data[["dentistry", "transcription", "medical_specialty"]]

data = data[["transcription"]]
data = data.reset_index(drop=True)
if __name__ == "__main__":
    print(data.reset_index(drop=True))



import pytest
# import os
# import sys
# from numpy import round as np_round
# from numpy import arange as np_arange
# sys.path.append(os.path.dirname(os.path.abspath(__file__)) + "/..")
# from AuDoLab import AuDoLab
# """Tests for `AuDoLab` package."""


@pytest.fixture
# def response():
#     """Sample pytest fixture.
#
#     See more at: http://doc.pytest.org/en/latest/fixture.html
#     """
#     import requests
#     return requests.get('https://github.com/audreyr/cookiecutter-pypackage')


def test_content(response):
    """Sample pytest test function with the pytest fixture as an argument."""
    # from bs4 import BeautifulSoup
    # assert 'GitHub' in BeautifulSoup(response.content).title.string


# audo = AuDoLab.AuDoLab()
# # papers = audo.scrape_abstracts(
# #     keywords=["dentistry", "teeth", "tooth"],
# #     in_data="all_meta",
# #     pages=12,
# #     operator="or"
# # )
# papers_processed = audo.preprocessing(papers, "text")
# data_processed = audo.preprocessing(data, "transcription")
#
# data_tfidf, papers_tfidf = audo.tf_idf( data_processed, papers_processed, data_column="lemma", papers_column="lemma")

# nus = np.arange(0.001, 0.5, 0.001)
# nus = np.round(nus, 5)
#
# classifier = audo.one_class_svm(
#     training=papers_tfidf,
#     predicting=data_tfidf,
#     nus=[0.166,0.3],
#     quality_train=0.85,
#     min_pred=0.01,
#     max_pred=0.1,
# )
#
# df_data = audo.choose_classifier(data_processed, classifier, 0)
#
# if __name__ == '__main__':
#     # lda = audo.lda_modeling(papers_processed, num_topics=3,
#     # random_state=101,
#     # passes=10,
#     # no_below=2, no_above=0.9
#     # )
#
#     lda2 = audo.lda_modeling(df_data, no_above=0.3)
#
#     a = audo.lda_visualize_topics()
#     html = a.data
#     with open('html_file.html', 'w') as f:
#         f.write(html)



from bs4 import BeautifulSoup
import requests
import pandas as pd
import re
from tqdm import tqdm


class AbstractScraper_Arxiv:
    def __init__(self):
        4 + 5

    def _find_links(self, url, number_of_pages):
        """finds paper links associated with given url search query

        Args:
            url (str): Link from arxiv.org with search query
            number_of_pages (int): number of pages that should be scraped
        """

        html = requests.get(url).text

        self.pages = []

        for page in BeautifulSoup(html, features="html.parser").findAll(
            "a", {"class": "pagination-link"}
        ):
            self.pages.append("https://arxiv.org" + page["href"])

        self.pages = list(set(self.pages))

        if len(self.pages) > number_of_pages:
            self.pages = self.pages[:number_of_pages]
        else:
            self.pages = self.pages

        self.document_links = []

        for page in self.pages:
            html = requests.get(page).text

            for link in BeautifulSoup(html, features="html.parser").find_all(
                "a", href=re.compile("/abs")
            ):
                self.document_links.append(link.get("href"))

        print(
            "The algorithm found ",
            len(self.document_links),
            " unique abstracts to in your query. \n Try increasing the number of pages if you want to scrape more papers",
        )

    def _scrape(self):
        """loop through all found paper links and scrape abstracts, titles and
        authors."""

        # initiliaze empty lists
        self.abstracts = []
        self.titles = []
        self.authors = []

        # loop through links and extract abstracts, titles and authors
        for url in tqdm(self.document_links):
            # for every link get underlying html code
            html = requests.get(url).text
            soup = BeautifulSoup(html, "html.parser")

            # use try (without specifiyng exception) for adapting to changes in source data
            # extract all abstracts with beautifulSoup
            try:
                abstract = soup.find("blockquote", class_="abstract")
                # extract the text between <span>
                abstract.span.extract()
                # replace \n (new lines)
                abstract = abstract.text.replace("\n", "")
                # append to data
                self.abstracts.append(abstract)
            except BaseException:
                self.abstracts.append(None)

            try:
                title = soup.find("h1", class_="title")
                title.span.extract()
                self.titles.append(title.text)
            except BaseException:
                self.titles.append(None)

            try:
                authors = soup.find("div", class_="authors")
                authors.span.extract()
                self.authors.append(authors.text)
            except BaseException:
                self.authors.append(None)

        data = pd.DataFrame(
            {
                "abstract": self.abstracts,
                "title": self.titles,
                "authors": self.authors,
            }
        )

        self.data = data.drop_duplicates()

    def scrape_arxiv(self, url, pages=8):
        """Scrapes arxiv.org and returns a pd.DataFrame containing abstracts,
            titles and author names.

        Returns these informations based on the users given url (search query),
        e.g. url="https://arxiv.org/search/?query=deep+learning&searchtype=all&source=header&order=&size=100&abstracts=show&date-date_type=submitted_date&start=0"

        Args:
            url (str): link of searchquery from arxiv.org
            pages (int, optional): number of pages the algorithm iterates
            through and searches for abstracts. Defaults to 8.

        Returns:
            pd.DataFrame: DataFrame that contains Abstracts, Titles and Authors
        """

        if "arxiv" not in url:
            return print(
                "ERROR: Only specify a url/search query via arxiv.org")
        else:
            self._find_links(url=url, number_of_pages=pages)
            self._scrape()

        return self.data


if __name__ == "__main__":
    AS = AbstractScraper_Arxiv()

    test = AS.scrape_arxiv(
        "https://arxiv.org/search/?query=deep+learning&searchtype=all&source=header&order=&size=100&abstracts=show&date-date_type=submitted_date&start=0",
        5,
    )



from sklearn.feature_extraction.text import TfidfVectorizer


class Tf_idf:

    def __init__(self):
        4 + 5

    @staticmethod
    def tfidf(
        data,
        papers,
        data_column="lemma",
        papers_column="lemma",
        features=None,
        ngrams=2,
    ):
        """Creates tf-idf objects for one-class SVM classification.

        The tf-idf scores are calculated over a joint corpus, however the target
        data and the out-of-domain training data are stored in seperate, as the
        one-class SVM is only trained on the tf-idf scores of the out-of-domain
        training data.

        Args:
            data (DataFrame): preprocessed target documents
            papers (DataFrame): preprocessed out-of-domain training data
            data_colum (String): name of columnin target dataframe where
                lemmatized documents are stored. Defaults to 'lemma'
            papers_colum (String): name of column in out-of-domain training
                dataframe where lemmatized documents are stored. Defaults to
            'lemma' ngrams (int, optional): whether ngram are formed.
                Defaults to 2.
            features (int, optional): number of max features.
                Defaults to 8000.

        Returns:
            data and papers: tfidf object data for target data and
                out-of-domain training data
        """

        df_temp = data.copy(deep=True)
        papers_temp = papers.copy(deep=True)

        tfidf_vectorizer = TfidfVectorizer(
            ngram_range=(1, ngrams), max_features=features
        )

        corpus = df_temp[data_column].tolist(
        ) + papers_temp[papers_column].tolist()
        tfidf_vectorizer.fit(corpus)

        data_corpus = df_temp[data_column].tolist()
        paper_corpus = papers_temp[papers_column].tolist()

        data = tfidf_vectorizer.transform(data_corpus)
        papers = tfidf_vectorizer.transform(paper_corpus)

        return data, papers



from bs4 import BeautifulSoup
import requests
import pandas as pd
import unicodedata
from tqdm import tqdm


class AbstractScraper_Pubmed:
    def __init__(self):
        pass

    def _find_links(self, url, number_of_pages):
        """finds paper links associated with given url search query

        Args:
            url (str): Link from arxiv.org with search query
            number_of_pages (int): number of pages that should be scraped.
        """

        html = requests.get(url).text

        self.pages = [url + "&page=" + str(i)
                      for i in range(1, number_of_pages)]

        self.pages = list(set(self.pages))

        self.document_links = []

        for page in self.pages:
            html = requests.get(page).text

            for link in BeautifulSoup(html, features="html.parser").findAll(
                "a", {"class": "docsum-title"}
            ):
                self.document_links.append(
                    "https://pubmed.ncbi.nlm.nih.gov" + link["href"]
                )

        print(
            "The algorithm found ",
            len(self.document_links),
            " unique abstracts to in your query. \n Try increasing the number of pages if you want to scrape more papers",
        )

    def _scrape(self):
        """loop through all found paper links and scrape abstracts, titles and
            authors."""

        # initiliaze empty lists
        self.abstracts = []
        self.titles = []
        self.authors = []

        # loop through links and extract abstracts, titles and authors
        for url in tqdm(self.document_links):
            # for every link get underlying html code
            html = requests.get(url).text
            soup = BeautifulSoup(html, "html.parser")

            # use try (without specifiyng exception) for adapting to changes in
            # source data
            # extract all abstracts with beautifulSoup
            try:
                abstract = soup.find("div", class_="abstract-content selected")
                # extract the text between <span>
                abstract = abstract.text.replace("\n", "")
                # append to data
                self.abstracts.append(abstract)
            except BaseException:
                self.abstracts.append(None)

            try:
                title = soup.find("h1", class_="heading-title")
                title = title.text.replace("\n", "")
                self.titles.append(title)
            except BaseException:
                self.titles.append(None)

            try:
                authors = soup.find("div", class_="authors")
                authors = authors.text.replace("\n", "")
                authors = unicodedata.normalize("NFKD", authors)
                authors = authors.replace("  ", "")
                authors = "".join(i for i in authors if not i.isdigit())
                self.authors.append(authors)
            except BaseException:
                self.authors.append(None)

        data = pd.DataFrame(
            {
                "abstract": self.abstracts,
                "title": self.titles,
                "authors": self.authors,
            }
        )

        self.data = data.drop_duplicates()

    def scrape_pubmed(self, url, pages=8):
        """Scrapes https://pubmed.ncbi and returns a pd.DataFrame containing
            abstracts, titles and author names.

        Returns these informations based on the users given url (search query),
        e.g. url="https://pubmed.ncbi.nlm.nih.gov/?term=medicine"

        Args:
            url (string): link of searchquery from arxiv.org
            pages (int, optional): number of pages the algorithm iterates
            through and searches for abstracts. Defaults to 8.

        Returns:
            pd.DataFrame: DataFrame that contains: Abstracts, Titles and Authors
        """

        if "pubmed" not in url:
            return print(
                "ERROR: Only specify a url/search query via https://pubmed.ncbi"
            )
        else:
            self._find_links(url=url, number_of_pages=pages)
            self._scrape()

        return self.data


if __name__ == "__main__":
    AS = AbstractScraper_Pubmed()

    test = AS.scrape_pubmed(
        "https://pubmed.ncbi.nlm.nih.gov/?term=medicine",
        3,
    )



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = []
with open("requirements.txt", "r") as fh:
    for line in fh:
        requirements.append(line.strip())

setup_requirements = [
    "pytest-runner",
]

test_requirements = [
    "pytest>=3",
]

setup(
    author="Arne Tillmann",
    author_email="arne.tillmann.vellmar@gmail.com",
    python_requires=">=3.5",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.8",
    ],
    description="With AuDoLab you can do LDA on highly imbalanced datasets.",
    entry_points={"console_scripts": ["AuDoLab=AuDoLab.cli:main", ], },
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme,
    include_package_data=True,
    keywords="AuDoLab",
    name="AuDoLab",
    packages=find_packages(include=["AuDoLab", "AuDoLab.*"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/ArneTillmann/AuDoLab",
    version='1.0.0',
    zip_safe=False,
)
